import ipaddress
import logging
from enrichr import dispatcher
datatype_logger = logging.getLogger('enrichr.config')


class BaseDataType:
    _raw = None
    _parsed = None
    regex = None
    defanged_regex = None
    stix_type = None
    metadata = {}
    common_name = None
    
    def __init__(self, data, defanged=False, *args, **kwargs):
        self._raw = data
        self._parsed = data
    
    @property
    def defang(self):
        return str(self._parsed)
    
    def parse(self):
        pass

    def refang(self):
        raise NotImplementedError('Refanging is not implemented for this data type')

    @property
    def is_valid(self):
        return True
    
    def enrich(self, *args, **kwargs):
        datatype_logger.debug(self)
        targets = dispatcher.get_enrichment_targets(self)
        datatype_logger.debug('would target the following enrichments: %s' % targets)

class IPv4Address(BaseDataType):
    regex = r'\b(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b'
    common_name = 'ip'

    def __init__(self, ip_address):
        self._raw = ip_address
        self.ipv4_address = ipaddress.IPv4Address(ip_address)

    @property
    def defang(self):
        str(self.ipv4_address).replace('.','[.]')




import logging
from logging import NullHandler
logging.getLogger(__name__).addHandler(NullHandler())

from enrichr.config import Configuration, ConfigOption
from enrichr.dispatch import Dispatcher


__VERSION__ = '0.0.4'

# Setup our global objects
dispatcher = Dispatcher()
config = Configuration()

# Define the global framework config options
Configuration.DEBUG = ConfigOption(name='debug', namespace='enrichr', default=False, help_text='Enable Debug Mode')
Configuration.LOGLEVEL = ConfigOption(name='loglevel', namespace='enrichr', default='CRITICAL', help_text='Log Level for Python Logging (defaults to Critical)')
Configuration.FORCE_PLUGIN_REGISTRATION = ConfigOption(
    name='force_plugin_registration',
    namespace='enrichr',
    default=False,
    help_text='Force Registration of Plugins (ignore namespace collisions)'
)




#!/usr/bin/env python3
from enrichr import config
from enrichr.exceptions import DuplicatePluginName
import logging

dispatch_logger = logging.getLogger('enrichr.dispatch')


class Dispatcher:
    _plugins = {}
    _enrichments = {}
    _default_enrichments = {}


    def register_plugin(self, plugin):
        plugin_name = plugin.__name__
        dispatch_logger.debug('Attempting to register plugin: %s' % plugin)
        if plugin_name not in self._plugins or config.FORCE_PLUGIN_REGISTRATION:
            dispatch_logger.debug('New registration of plugin: %s' % plugin)
            self._plugins[plugin_name] = plugin
        elif plugin_name in self._plugins and plugin is not self._plugins[plugin_name]:
            raise DuplicatePluginName('Plugin name collision')
        return True

    def register_enrichment(self, func, datatype):
        dispatch_logger.debug('Attempting to register the enrichment: %s for %s' % (func, datatype))
        return True
    
    def get_enrichment_targets(self, datatype):
        if datatype in self._enrichments:
            return self._enrichments[datatype]
    
    def reset(self):
        self._plugins = {}
        self._enrichments = {}




#!/usr/bin/env python3
import logging
from inspect import isclass
from functools import wraps
from enrichr import dispatcher
from enrichr import config
from enrichr.datatypes import BaseDataType

plugin_logger = logging.getLogger('enrichr.plugin')

def enrichment(datatype, default=False, bulk=False, *args, **kwargs):
    plugin_logger.debug('enrichment decorator: %s, %s, %s' % (datatype, args, kwargs))
    if not isclass(datatype) or not issubclass(datatype, BaseDataType):
        raise TypeError("@enrichment must be passed a valid datatype (was '%s'), for example @enrichment(IPv4Address)" % datatype)
    def _enrichment_decorator(func):
        # Register the function with the dispatcher
        dispatcher.register_enrichment(func, datatype)
        @wraps(func)
        def _enrichment_wrapped_function(*f_args, **f_kwargs):
            plugin_logger.debug('Function Args/Kwargs: %s, %s' % (f_args, f_kwargs))
            plugin_logger.debug('Parent Args/Kwargs: %s, %s' % (args, kwargs))
            return func(*f_args, **f_kwargs)
        return _enrichment_wrapped_function
    return _enrichment_decorator

def autoload_plugins():
    pass

class Plugin:
    namespace = None
    config = config
    version = None
    plugin_license = None
    maintainer = None
    description = None
    meta = {}  # License, Author, Description, etc.

    def __init_subclass__(cls, /, **kwargs):
        # Register the plugin as being loaded
        dispatcher.register_plugin(cls)
        if hasattr(cls, 'load_plugin'):
            cls.load_plugin()
        super().__init_subclass__()

    @property
    def configured(self):
        missing_attrs = []
        # Check required config
        for attr in ['version', 'license', 'maintainer', 'description']:
            if getattr(self, attr) is not None:
                missing_attrs.append(attr)
        if missing_attrs:
            plugin_logger.critical('Plugin (%s) is not configured correctly, missing: %s' % (self.__qualname__, ','.join(missing_attrs)))
            return False
        # We should use the self.config object to verify that everything is configured for the namespace we are operating in.
        return True
    
    @classmethod
    def load_plugin(cls):
        pass





#!/usr/bin/env python3

class NotConfiguredError(Exception):
    """The requested configuration item has not yet been configured"""
    pass
    
class DuplicatePluginName(Exception):
    """Plugin name isn't unique"""
    pass



import re
import os
import logging
from enrichr.exceptions import NotConfiguredError

ENV_PREFIX = 'ENRICHR'
ENRICHR_NAMESPACE = 'ENRICHR'
ATTR_DELIMETER = '__'
config_logger = logging.getLogger('enrichr.config')


def valid_name(name):
    # Starts with a letter, and contains uppercase, numbers or _
    name_re = re.compile(r'^[A-Z][A-Z0-9_]*$')
    if name_re.match(name):
        return True
    else:
        return False

class Configuration(object):
    _config_values = {}
    _masked_values = []
    
    @classmethod
    def register_option(cls, option):
        # When we register options, we need to ensure that:
        #   - the objects namespace and name is present
        #   - The option is a ConfigOption
        if not isinstance(option, ConfigOption):
            raise Exception('ConfigOption is not a valid config.ConfigOption')
        # If the option exists, then update with the new option
        if hasattr(cls, option.attr_name):
            object.__getattribute__(cls, option.attr_name).update(option)
            return True
        setattr(cls, option.attr_name, option)
        return True

    @classmethod
    def mask_value(cls, value):
        # This is used for hiding sensitive fields from logging
        if value not in cls._masked_values:
            cls._masked_values.append(value)


class ConfigOption:
    _registered = False
    def __init__(self, name=None, namespace=None, default=None, env_var=None, help_text=None, mask=False, order=0, required=False, **kwargs):
        if name:
            name = name.upper()
            if not valid_name(name):
                raise ValueError("ConfigOption 'name' (%s) must start with a letter and only contain letters, numbers and underscores (_)" % name)
        self.name = name
        if namespace:
            namespace = namespace.upper()
            if not valid_name(namespace):
                raise ValueError("ConfigOption 'namespace' (%s) must start with a letter and only contain letters, numbers and underscores (_)" % namespace)
        self.namespace = namespace
        if required and default:
            raise ValueError("ConfigOption arguments 'required' (%s) and 'default' (%s) cannot both be set" % (repr(required), repr(default)))
        self.required = required
        self.default = default
        self._env_var = env_var
        self.help_text = help_text
        self.mask = mask
        self.order = order
        # Hit the registered property in case we have everything thats needed
        self.register()
        return
    
    def register(self):
        if self._registered:
            return True
        if self.namespace and self.name:
            Configuration.register_option(self)
            self._registered = True
            return True
        return False

    def update(self, option):
        option_attrs = vars(option)
        for attr_name, value in option_attrs.items():
            if value is not None:
                setattr(self, attr_name, value)

    @property
    def attr_name(self):
        if self.namespace == ENRICHR_NAMESPACE:
            return self.name
        return self.namespace + ATTR_DELIMETER + self.name

    @property
    def env_var(self):
        if self._env_var:
            return self._env_var
        return ENV_PREFIX + ATTR_DELIMETER + self.attr_name

    def __set_name__(self, obj, name):
        # If the namespace hasn't been defined, get the namespace from the object which is is defining the ConfigOption
        if not self.namespace:
            # If the object that is using the ConfigurationOption has a namespace attribute set, then use that as the namespace
            if hasattr(obj, 'namespace') and obj.namespace:
                self.namespace = obj.namespace.upper()
            # Otherwise, we will set the namespace to be the class name of the object defining the ConfigOption
            else:
                self.namespace = obj.__name__.upper()
        # If there isn't a name defined, get it from the name of the variable that ConfigObject is being set to
        if not self.name:
            self.name = name.upper()
        # Register the configuration option with the Configuration class if we haven't already
        self.register()

    def __get__(self, obj, objtype=None):
        # If we aren't working within an instance and it is a Configuration object, allow it to be worked with directly.
        if obj is None and objtype is Configuration:
            return self
        # Before we do anything, make sure the descriptor is registered
        if not self.register():
            raise AttributeError("Both the 'name' (%s) and 'namespace' (%s) are required in ConfigOption before using '__get__'" % (self.name, self.namespace))
        if hasattr(obj, '_config_values') and self.attr_name in obj._config_values:
            return obj._config_values[self.attr_name]
        # This is typically a plugin that has a .config attribute, we will verify there is a ._config_values in there and grab that if it exists.
        if hasattr(obj, 'config') and hasattr(obj.config, '_config_values') and self.attr_name in obj.config._config_values:
            return obj.config._config_values[self.attr_name]
        # If we have an environmental variable, then use that
        if self.env_var in os.environ:
            return os.environ[self.env_var]
        # If the option is required, we need to raise an exception
        if self.required:
            raise NotConfiguredError("A required configuration item has not been set for 'config.%s'" % self.attr_name)
        # Just return the default value
        return self.default

    def __set__(self, obj, value):
        # Before we do anything, make sure the descriptor is registered
        if not self.register():
            raise AttributeError("Both the 'name' (%s) and 'namespace' (%s) are required in ConfigOption before using '__set__'" % (self.name, self.namespace))
        # If the ConfigurationOption is supposed to be masked (for example in logs) use the Configuration.mask_values to add it to our global masks
        if self.mask:
            Configuration.mask_value(value)
        # We will always set the value to the objects obj._config_value.
        if not hasattr(obj, '_config_values'):
            obj._config_values = {}
        obj._config_values[self.attr_name] = value
        

